@@ -4,7 +4,7 @@ import time
 import atexit
 import json
 
-egun_version = "0.0.37 (Sat, 2022-11-24)"
+egun_version = "0.0.38 (Sat, 2022-11-29)"
 
 print(f"Electron source controller: {egun_version}")
 
@@ -143,7 +143,7 @@ class ElectronGunControl:
     POLARITY_POS = b'p'
     POLARITY_NEG = b'n'
 
-    def __init__(self, portFile = None, commandRetries = 3, shutdownOnTerminate = True):
+    def __init__(self, portFile = None, commandRetries = 3, shutdownOnTerminate = False):
         self._shutdownOnTerminate = shutdownOnTerminate
         if not portFile:
             portFile = '/dev/ttyU0'
@@ -810,6 +810,189 @@ class ElectronGunControl:
             else:
                 return None
 
+    def getTargetVoltages(self, *ignore, sync = False):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        self.port.write(b'$$$getvtarget\n')
+        self._lastcommand = b'$$$getvtarget\n'
+        if sync:
+            return self.internal__waitForMessageFilter("vtargets")
+        else:
+            return None
+
+    def getCurrentLimitBeam(self, *ignore, sync = False):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        self.port.write(b'$$$getbeamcurlim\n')
+        self._lastcommand = b'$$$getbeamcurlim\n'
+        if sync:
+            return self.internal__waitForMessageFilter("beamcurlim")
+        else:
+            return None
+
+    def getCurrentLimitInsulation(self, *ignore, sync = False):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        self.port.write(b'$$$getinsulcurlim\n')
+        self._lastcommand = b'$$$getinsulcurlim\n'
+        if sync:
+            return self.internal__waitForMessageFilter("insulcurlim")
+        else:
+            return None
+
+    def getRampSteps(self, *ignore, sync = False):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        self.port.write(b'$$$getstepsizes\n')
+        self._lastcommand = b'$$$getstepsizes\n'
+        if sync:
+            return self.internal__waitForMessageFilter("getstepsizes")
+        else:
+            return None
+
+    def getRampDurations(self, *ignore, sync = False):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        self.port.write(b'$$$getdurations\n')
+        self._lastcommand = b'$$$getdurations\n'
+        if sync:
+            return self.internal__waitForMessageFilter("getdurations")
+        else:
+            return None
+
+    def setTargetVoltage(self, *ignore, cathode = None, wehnelt = None, focus = None, aux = None):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        if cathode is not None:
+            if (int(cathode) < 0) or (int(cathode) > 3250):
+                raise ValueError("Cathode voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setvtargetvk'
+            cmd = bytes(str(cathode), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if wehnelt is not None:
+            if (int(wehnelt) < 0) or (int(wehnelt) > 3250):
+                raise ValueError("Wehnelt voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setvtargetvw'
+            cmd = bytes(str(wehnelt), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if focus is not None:
+            if (int(focus) < 0) or (int(focus) > 3250):
+                raise ValueError("Focus voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setvtargetvf'
+            cmd = bytes(str(focus), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if aux is not None:
+            if (int(aux) < 0) or (int(aux) > 3250):
+                raise ValueError("Aux voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setvtargetva'
+            cmd = bytes(str(aux), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        time.sleep(5)
+
+    def setCurrentLimitBeam(self, *ignore, cathode = None, wehnelt = None, focus = None, aux = None):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        if cathode is not None:
+            if (int(cathode) < 0) or (int(cathode) > 3250):
+                raise ValueError("Cathode voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setbeamcurlimk'
+            cmd = bytes(str(cathode), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if wehnelt is not None:
+            if (int(wehnelt) < 0) or (int(wehnelt) > 3250):
+                raise ValueError("Wehnelt voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setbeamcurlimw'
+            cmd = bytes(str(wehnelt), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if focus is not None:
+            if (int(focus) < 0) or (int(focus) > 3250):
+                raise ValueError("Focus voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setbeamcurlimf'
+            cmd = bytes(str(focus), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if aux is not None:
+            if (int(aux) < 0) or (int(aux) > 3250):
+                raise ValueError("Aux voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setbeamcurlima'
+            cmd = bytes(str(aux), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        time.sleep(5)
+
+    def setCurrentLimitInsulation(self, *ignore, cathode = None, wehnelt = None, focus = None, aux = None):
+        if self.port == False:
+            raise ElectronGunNotConnected("Electron gun currently not connected")
+        if cathode is not None:
+            if (int(cathode) < 0) or (int(cathode) > 3250):
+                raise ValueError("Cathode voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setinsulcurlimk'
+            cmd = bytes(str(cathode), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if wehnelt is not None:
+            if (int(wehnelt) < 0) or (int(wehnelt) > 3250):
+                raise ValueError("Wehnelt voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setinsulcurlimw'
+            cmd = bytes(str(wehnelt), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if focus is not None:
+            if (int(focus) < 0) or (int(focus) > 3250):
+                raise ValueError("Focus voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setinsulcurlimf'
+            cmd = bytes(str(focus), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if aux is not None:
+            if (int(aux) < 0) or (int(aux) > 3250):
+                raise ValueError("Aux voltage has to be in range from 0 to 3250")
+            cmd = b'$$$setinsulcurlima'
+            cmd = bytes(str(aux), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        time.sleep(5)
+
+    def setRampVoltageStepSize(self, vstep):
+        if (int(vstep) < 0) or (int(vstep) > 200):
+            raise ValueError("Voltage step size is our of range")
+        self.port.write(b'$$$setstepsizev' + bytes(str(vstep), encoding="ascii") + b'\n')
+        time.sleep(1)
+    def setRampVoltageFilamentCurrent(self, istep):
+        if (int(istep) < 0) or (int(istep) > 200):
+            raise ValueError("Current step size is out of range")
+        self.port.write(b'$$$setstepsizeila' + bytes(str(istep), encoding="ascii") + b'\n')
+
+    def setRampDurations(self, *ignore, initDuration = None, voltageStep = None, filamentStep = None):
+        if (initDuration is not None) and (initDuration <= 0):
+            raise ValueError("Init duration is out of range")
+        if (voltageStep is not None) and (voltageStep <= 0):
+            raise ValueError("Voltage stepsize is out of range")
+        if (filamentStep is not None) and (filamentStep <= 0):
+            raise ValueError("Filament stepsize is out of range")
+
+        if initDuration is not None:
+            cmd = b'$$$setdurationinit'
+            cmd = bytes(str(initDuration), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if voltageStep is not None:
+            cmd = b'$$$setdurationstepv'
+            cmd = bytes(str(voltageStep), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+        if filamentStep is not None:
+            cmd = b'$$$setdurationstepfila'
+            cmd = bytes(str(filamentStep), encoding="ascii")
+            cmd = cmd + b'\n'
+            self.port.write(cmd)
+
     def jabber(self, *ignore, sync = False):
         if self.port == False:
             raise ElectronGunNotConnected("Electron gun currently not connected")
@@ -853,4 +1036,4 @@ class ElectronGunControl:
         return s
 
     def __repr__(self):
-        return self.get_status()
+        return "Electron gun controller {egun_version}"
